@@ -1,16 +1,18 @@
 # -*- coding: utf-8 -*-
 """RainCloud Faucet."""
+import time
+
 from raincloudy.const import (
     HOME_ENDPOINT, MANUAL_OP_DATA, MANUAL_WATERING_ALLOWED,
-    MAX_RAIN_DELAY_DAYS, MAX_WATERING_MINUTES)
+    MAX_RAIN_DELAY_DAYS, MAX_WATERING_MINUTES, HEADERS, STATUS_ENDPOINT)
 from raincloudy.helpers import (
-    find_controller_or_faucet_name, find_zone_name)
+    find_controller_or_faucet_name, find_selected_controller_or_faucet_index)
 
 
 class RainCloudyFaucetCore():
     """RainCloudyFaucetCore object."""
 
-    def __init__(self, parent, controller, faucet_id):
+    def __init__(self, parent, controller, faucet_id, index, zone_names=None):
         """
         Initialize RainCloudy Controller object.
 
@@ -23,10 +25,15 @@ class RainCloudyFaucetCore():
         :return: RainCloudyFaucet object
         :rtype: RainCloudyFaucet object
         """
+        if zone_names is None:
+            zone_names = []
 
+        self.index = index
         self._parent = parent
         self._controller = controller
         self._id = faucet_id
+        self._attributes = {}
+        self._zone_names = zone_names
 
         # zones associated with faucet
         self.zones = []
@@ -42,7 +49,8 @@ class RainCloudyFaucetCore():
                     parent=self._parent,
                     controller=self._controller,
                     faucet=self,
-                    zone_id=zone_id)
+                    zone_id=zone_id,
+                    zone_name=self._zone_names[zone_id - 1])
 
             if zone not in self.zones:
                 self.zones.append(zone)
@@ -55,9 +63,9 @@ class RainCloudyFaucetCore():
             return "<{0}: {1}>".format(self.__class__.__name__, self.id)
 
     @property
-    def _attributes(self):
-        """Callback to self._controller attributes."""
-        return self._controller.attributes
+    def attributes(self):
+        """Return faucet id."""
+        return self._attributes
 
     @property
     def serial(self):
@@ -80,18 +88,20 @@ class RainCloudyFaucetCore():
         """Return faucet name."""
         return \
             find_controller_or_faucet_name(
-                self._parent.html['home'],
-                'faucet')
+                self._controller.home,
+                'faucet',
+                self.index
+            )
 
     @name.setter
     def name(self, value):
         """Set a new name to faucet."""
         data = {
             '_set_faucet_name': 'Set Name',
-            'select_faucet': 0,
+            'select_faucet': self.index,
             'faucet_name': value,
         }
-        self._controller.post(data)
+        self._parent.post(data)
 
     @property
     def status(self):
@@ -107,8 +117,28 @@ class RainCloudyFaucetCore():
         return battery.strip('%')
 
     def update(self):
-        """Callback self._controller.update()."""
-        self._controller.update()
+        """Submit GET request to update information."""
+        # adjust headers
+        headers = HEADERS.copy()
+        headers['Accept'] = '*/*'
+        headers['X-Requested-With'] = 'XMLHttpRequest'
+        headers['X-CSRFToken'] = self._parent.csrftoken
+
+        args = '?controller_serial=' + self._controller.serial \
+               + '&faucet_serial=' + self.id
+
+        req = self._parent.client.get(STATUS_ENDPOINT + args,
+                                      headers=headers)
+
+        # token probably expired, then try again
+        if req.status_code == 403:
+            self._parent.login()
+            self.update()
+        elif req.status_code == 200:
+            self._attributes = req.json()
+            self._controller.attributes = self._attributes
+        else:
+            req.raise_for_status()
 
     def _find_zone_by_id(self, zone_id):
         """Return zone by id."""
@@ -150,7 +180,7 @@ class RainCloudyFaucetZone(RainCloudyFaucetCore):
 
     # pylint: disable=super-init-not-called
     # needs review later
-    def __init__(self, parent, controller, faucet, zone_id):
+    def __init__(self, parent, controller, faucet, zone_id, zone_name):
         """
         Initialize RainCloudy Controller object.
 
@@ -169,6 +199,7 @@ class RainCloudyFaucetZone(RainCloudyFaucetCore):
         self._controller = controller
         self._faucet = faucet
         self._id = zone_id
+        self._name = zone_name
 
     def __repr__(self):
         """Object representation."""
@@ -182,16 +213,18 @@ class RainCloudyFaucetZone(RainCloudyFaucetCore):
         # zone starts with index 0
         zoneid -= 1
         data = {
+            'select_controller': self._controller.index,
+            'select_faucet': self._faucet.index,
             '_set_zone_name': 'Set Name',
             'select_zone': str(zoneid),
             'zone_name': name,
         }
-        self._controller.post(data)
+        self._parent.post(data)
 
     @property
     def name(self):
         """Return zone name."""
-        return find_zone_name(self._parent.html['home'], self.id)
+        return self._name
 
     @name.setter
     def name(self, value):
@@ -206,15 +239,21 @@ class RainCloudyFaucetZone(RainCloudyFaucetCore):
                     ', '.join(map(str, MANUAL_WATERING_ALLOWED)))
             )
 
+        ddata = self.preupdate()
+        attr = 'zone{}_select_manual_mode'.format(zoneid)
+
         if isinstance(value, int) and value == 0:
             value = 'OFF'
+
+            # If the zone is turned on at the valve we need to toggle ON before we can shut OFF
+            ddata[attr] = 'ON'
+            self.submit_action(ddata)
+            time.sleep(1)
         elif isinstance(value, str):
             value = value.upper()
             if value == 'ON':
                 value = MAX_WATERING_MINUTES
 
-        ddata = self.preupdate()
-        attr = 'zone{}_select_manual_mode'.format(zoneid)
         ddata[attr] = value
         self.submit_action(ddata)
 
@@ -298,6 +337,7 @@ class RainCloudyFaucetZone(RainCloudyFaucetCore):
                 ddata[attr] = 'on'
         except KeyError:
             pass
+
         self.submit_action(ddata)
         return True
 
@@ -318,7 +358,8 @@ class RainCloudyFaucetZone(RainCloudyFaucetCore):
 
     def lookup_attr(self, attr):
         """Returns rain_delay_mode attributes by zone index"""
-        return self._attributes['rain_delay_mode'][int(self.id) - 1][attr]
+        return self._faucet.attributes['rain_delay_mode'][int(self.id) - 1][
+            attr]
 
     def _to_dict(self):
         """Method to build zone dict."""
@@ -343,13 +384,17 @@ class RainCloudyFaucetZone(RainCloudyFaucetCore):
         """Return status from zone."""
         return self._to_dict()
 
+    def update(self):
+        """Request faucet to update"""
+        return self._faucet.update()
+
     def preupdate(self, force_refresh=True):
         """Return a dict with all current options prior submitting request."""
         ddata = MANUAL_OP_DATA.copy()
 
         # force update to make sure status is accurate
         if force_refresh:
-            self.update()
+            self._faucet.update()
 
         # select current controller and faucet
         ddata['select_controller'] = \
@@ -386,6 +431,30 @@ class RainCloudyFaucetZone(RainCloudyFaucetCore):
 
     def submit_action(self, ddata):
         """Post data."""
-        self._controller.post(ddata,
+
+        controller_index = self._parent.controllers.index(self._controller)
+        faucet_index = self._controller.faucets.index(self._faucet)
+
+        current_controller_index = find_selected_controller_or_faucet_index(
+            self._parent.html['home'], 'controller')
+
+        current_faucet_index = find_selected_controller_or_faucet_index(
+            self._parent.html['home'], 'faucet')
+
+        # This is an artifact of how the web-page we're impersonating works.
+        # The form submit will only apply actions to _selected_ controllers
+        # and faucets. So if the active controller and/or faucet on the page
+        # isn't the faucet we're trying to submit an action for we need to
+        # send the response twice. The first time we send it will switch us
+        # to the action
+        if current_controller_index != controller_index or \
+                current_faucet_index != faucet_index:
+            self._parent.post(ddata,
                               url=HOME_ENDPOINT,
                               referer=HOME_ENDPOINT)
+
+        response = self._parent.post(ddata,
+                                     url=HOME_ENDPOINT,
+                                     referer=HOME_ENDPOINT)
+
+        self._parent.update_home(response.text)
