@@ -7,24 +7,23 @@ from raincloudy.exceptions import RainCloudyException
 def generate_soup_html(data):
     """Return an BeautifulSoup HTML parser document."""
     try:
-        return BeautifulSoup(data, 'html.parser')
+        return BeautifulSoup(data, 'html5lib')
     except:
         raise TypeError(
             'Invalid data passed to BeautifulSoup')
 
 
-def serial_finder(data):
+def faucet_serial_finder(data):
     """
-    Find controller serial and faucet_serial from the setup page.
+    Find faucet_serial from the setup page.
 
     <select id="id_select_controller2" name="select_controller" >
         <option value='0' selected='selected'>1 - Controller001</option>
+        <option value='1>2 - Controller002</option>
     </select>
 
-    :param data: text to be parsed
-    :type data: BeautilSoup object
-    :return: a dict with controller_serial and faucet_serial
-    :rtype: dict
+    :param data: text to be parsed :type data: BeautilSoup object :return: a
+    dict with array of controller_serials and array of faucet_serials
     :raises IndexError: if controller_serial was not found on the data
     """
     if not isinstance(data, BeautifulSoup):
@@ -32,30 +31,52 @@ def serial_finder(data):
 
     try:
 
-        # The setup page contains a select box for each controller and each
-        # faucet
-        controllers_element = data.find_all('select',
-                                            {'id': 'id_select_controller2'})
+        faucets_element = data.find(id="id_select_faucet2").find_all('option')
 
-        faucets_element = data.find_all('select',
-                                        {'id': 'id_select_faucet2'})
+        faucet_serials = []
+        for faucet_element in faucets_element:
+            faucet_serials.append(faucet_element.text.split('-')[1].strip())
 
-        controller_serial = controllers_element[0].text.split('-')[1].strip()
-        faucet_serial = faucets_element[0].text.split('-')[1].strip()
-
-        # currently only one faucet is supported on the code
-        # we have plans to support it in the future
-        parsed_dict = {}
-        parsed_dict['controller_serial'] = controller_serial
-        parsed_dict['faucet_serial'] = [faucet_serial]
-        return parsed_dict
+        return faucet_serials
 
     except (AttributeError, IndexError, ValueError):
         raise RainCloudyException(
             'Could not find any valid controller or faucet')
 
 
-def find_controller_or_faucet_name(data, p_type):
+def controller_serial_finder(data):
+    """
+    Find all controller serials from the setup page.
+
+    <select id="id_select_controller2" name="select_controller" >
+        <option value='0' selected='selected'>1 - Controller001</option>
+    </select>
+
+    :param data: text to be parsed
+    :type data: BeautilSoup object
+    :return: an array of controller serials
+    :raises IndexError: if controller_serial was not found on the data
+    """
+
+    try:
+
+        controllers_element = data.find(id="id_select_controller2").find_all(
+            'option')
+
+        controller_serials = []
+
+        for controller_element in controllers_element:
+            controller_serials.append(
+                controller_element.text.split('-')[1].strip())
+
+        return controller_serials
+
+    except (AttributeError, IndexError, ValueError):
+        raise RainCloudyException(
+            'Could not find any valid controller serials')
+
+
+def find_controller_or_faucet_name(data, p_type, index=0):
     """
     Find on the HTML document the controller name.
 
@@ -67,6 +88,7 @@ def find_controller_or_faucet_name(data, p_type):
        name="select_controller" onchange="submit()" >
           <option value="0" selected="selected">HERE_IS_CONTROLLER_NAME
 
+    :param index: The index of the element we're parsing
     :param data: BeautifulSoup object
     :param p_type: parameter type. (controller or faucet)
     :return: controller or valve name
@@ -82,13 +104,16 @@ def find_controller_or_faucet_name(data, p_type):
 
     try:
         search_field = 'id_select_{0}'.format(p_type)
-        child = data.find('select', {'id': search_field})
-        return child.get_text().strip()
+        child = data.find(id=search_field).findAll('option')
+
+        parsed_text = child[index].text.strip()
+
+        return parsed_text
     except AttributeError:
         return None
 
 
-def find_zone_name(data, zone_id):
+def find_zone_names(data):
     """
     Find on the HTML document the zone name.
 
@@ -97,7 +122,6 @@ def find_zone_name(data, zone_id):
         title="Zone can be renamed on Setup tab">1 - zone1</span>,
 
     :param data: BeautifulSoup object
-    :param zone: zone id
     :return: zone name
     :rtype: string
     :raises TypeError: if data is not a BeautifulSoup object
@@ -106,13 +130,52 @@ def find_zone_name(data, zone_id):
     if not isinstance(data, BeautifulSoup):
         raise TypeError("Function requires BeautilSoup HTML element.")
 
-    table = data.find('table', {'class': 'zone_table'})
-    table_body = table.find('tbody')
-    rows = table_body.find_all('span', {'class': 'more_info'})
-    for row in rows:
-        if row.get_text().startswith(str(zone_id)):
-            return row.get_text()[4:].strip()
-    return None
+    try:
+        zones = data.find('select', {'name': 'select_zone'}).find_all(
+            'option')
+    except AttributeError:
+        return ['1', '2', '3', '4']
+
+    zone_names = []
+
+    for zone in zones:
+        raw_name_array = zone.text.split('-')
+        if len(raw_name_array) > 1:
+            zone_names.append(raw_name_array[1].strip())
+        else:
+            zone_names.append('')
+
+            # This is a breaking change for downstream
+            # zone_names.append(f"Zone: {zone.text.strip()}")
+
+    return zone_names
 
 
+def find_selected_controller_or_faucet_index(data, p_type):
+    """
+    Find the currently selected controller index from the home html
+    :param p_type: parameter type. (controller or faucet)
+    :param data: BeautifulSoup object
+    :return: controller index
+    """
+
+    entity_index = None
+    if not isinstance(data, BeautifulSoup):
+        raise TypeError("Function requires BeautilSoup HTML element.")
+
+    if p_type not in ('controller', 'faucet'):
+        raise TypeError("Function p_type must be controller or faucet")
+
+    try:
+        search_field = 'id_select_{0}'.format(p_type)
+        child = data.find(id=search_field).findAll('option')
+
+        for index, option in enumerate(child):
+            if "selected" in str(option):
+                entity_index = index
+
+    except AttributeError:
+        pass
+
+    return entity_index
 # vim:sw=4:ts=4:et:
